@@ -4,8 +4,7 @@ use crate::sessionwrap::SessionWrap;
 use filedescriptor::{socketpair, FileDescriptor};
 use portable_pty::{ExitStatus, PtySize};
 use smol::channel::{bounded, Receiver, TryRecvError};
-use std::collections::HashMap;
-use std::collections::VecDeque;
+use std::collections::{HashMap, VecDeque};
 use std::io::{Read, Write};
 use std::sync::Mutex;
 
@@ -126,17 +125,6 @@ impl portable_pty::Child for SshChildProcess {
         }
     }
 
-    fn kill(&mut self) -> std::io::Result<()> {
-        if let Some(tx) = self.tx.as_ref() {
-            tx.try_send(SessionRequest::SignalChannel(SignalChannel {
-                channel: self.channel,
-                signame: "HUP",
-            }))
-            .map_err(|e| std::io::Error::new(std::io::ErrorKind::Other, e.to_string()))?;
-        }
-        Ok(())
-    }
-
     fn wait(&mut self) -> std::io::Result<portable_pty::ExitStatus> {
         if let Some(status) = self.exited.as_ref() {
             return Ok(status.clone());
@@ -164,6 +152,52 @@ impl portable_pty::Child for SshChildProcess {
     }
 }
 
+impl portable_pty::ChildKiller for SshChildProcess {
+    fn kill(&mut self) -> std::io::Result<()> {
+        if let Some(tx) = self.tx.as_ref() {
+            tx.try_send(SessionRequest::SignalChannel(SignalChannel {
+                channel: self.channel,
+                signame: "HUP",
+            }))
+            .map_err(|e| std::io::Error::new(std::io::ErrorKind::Other, e.to_string()))?;
+        }
+        Ok(())
+    }
+
+    fn clone_killer(&self) -> Box<dyn portable_pty::ChildKiller + Send + Sync> {
+        Box::new(SshChildKiller {
+            tx: self.tx.clone(),
+            channel: self.channel,
+        })
+    }
+}
+
+#[derive(Debug, Clone)]
+struct SshChildKiller {
+    pub(crate) tx: Option<SessionSender>,
+    pub(crate) channel: ChannelId,
+}
+
+impl portable_pty::ChildKiller for SshChildKiller {
+    fn kill(&mut self) -> std::io::Result<()> {
+        if let Some(tx) = self.tx.as_ref() {
+            tx.try_send(SessionRequest::SignalChannel(SignalChannel {
+                channel: self.channel,
+                signame: "HUP",
+            }))
+            .map_err(|e| std::io::Error::new(std::io::ErrorKind::Other, e.to_string()))?;
+        }
+        Ok(())
+    }
+
+    fn clone_killer(&self) -> Box<dyn portable_pty::ChildKiller + Send + Sync> {
+        Box::new(SshChildKiller {
+            tx: self.tx.clone(),
+            channel: self.channel,
+        })
+    }
+}
+
 impl crate::sessioninner::SessionInner {
     pub fn new_pty(
         &mut self,
