@@ -1,5 +1,6 @@
 use crate::sshd::session;
-use assert_fs::{prelude::*, TempDir};
+use assert_fs::prelude::*;
+use assert_fs::TempDir;
 use predicates::prelude::*;
 use rstest::*;
 use std::convert::TryInto;
@@ -612,7 +613,13 @@ async fn canonicalize_should_either_return_resolved_path_or_error_if_missing(
     match result {
         Ok(_) => {}
         Err(SftpChannelError::Sftp(SftpError::NoSuchFile)) => {}
-        x => panic!("Unexpected result from canonicalize: {:?}", x),
+        #[cfg(feature = "libssh-rs")]
+        Err(SftpChannelError::LibSsh(libssh_rs::Error::Sftp(_))) => {}
+        x => panic!(
+            "Unexpected result from canonicalize({}: {:?}",
+            missing.path().display(),
+            x
+        ),
     }
 }
 
